# Builtins

- **`toString`:** Converts a value to its string representation.
   ```nix
   builtins.toString 42  # Output: "42"
   ```

- **`length`:** Returns the number of elements in a list or string.
   ```nix
   builtins.length [1 2 3]  # Output: 3
   builtins.length "Hello"  # Output: 5
   ```

- **`elem`:** Checks if an element is present in a list or string.
   ```nix
   builtins.elem 2 [1 2 3]  # Output: true
   builtins.elem "o" "Hello"  # Output: true
   ```

- **`head`:** Returns the first element of a list.
   ```nix
   builtins.head [1 2 3]  # Output: 1
   ```

- **`tail`:** Returns all but the first element of a list.
   ```nix
   builtins.tail [1 2 3]  # Output: [2 3]
   ```

- **`map`:** Applies a function to each element of a list.
   ```nix
   builtins.map (x: x * 2) [1 2 3]  # Output: [2 4 6]
   ```

- **`filter`:** Filters a list based on a predicate function.
   ```nix
   builtins.filter (x: x > 2) [1 2 3]  # Output: [3]
   ```

- **`foldl`:** Performs a left fold on a list.
   ```nix
   builtins.foldl (x: y: x + y) 0 [1 2 3]  # Output: 6
   ```

- **`concatLists`:** Concatenates multiple lists into a single list.
   ```nix
   builtins.concatLists [[1 2] [3 4]]  # Output: [1 2 3 4]
   ```

- **`split`:** Splits a string into a list of substrings based on a delimiter.
    ```nix
    builtins.split ":" "foo:bar:baz"  # Output: ["foo" "bar" "baz"]
    ```

- **`elemAt`:** Retrieves the element at a specific index in a list.
    ```nix
    builtins.elemAt 1 [10 20 30]  # Output: 20
    ```

- **`intersectAttrs`:** Finds the intersection of attributes in two sets.
    ```nix
    builtins.intersectAttrs { a = 1; b = 2; } { b = 2; c = 3; }  # Output: { b = 2; }
    ```

- **`sub`:** Subtracts one integer from another.
    ```nix
    builtins.sub 10 5  # Output: 5
    ```

- **`mul`:** Multiplies two integers.
    ```nix
    builtins.mul 5 2  # Output: 10
    ```

- **`div`:** Divides one integer by another.
    ```nix
    builtins.div 10 2  # Output: 5
    ```

- **`mod`:** Returns the remainder of dividing one integer by another.
    ```nix
    builtins.mod 10 3  # Output: 1
    ```

- **`parseDrvName`:** Parses a Nix store path and returns a derivation name.
    ```nix
    builtins.parseDrvName "/nix/store/abcdefg.drv"  # Output: "abcdefg"
    ```

- **`toString`:** Converts a value to its string representation.
    ```nix
    builtins.toString true  # Output: "true"
    ```
